# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: example.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'example.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rexample.proto\x12\x0f\x65xample_service\"`\n\x0e\x45xampleRequest\x12%\n\x0e\x65xample_string\x18\x01 \x01(\tR\rexampleString\x12\'\n\x0f\x65xample_integer\x18\x02 \x01(\x03R\x0e\x65xampleInteger\"a\n\x0f\x45xampleResponse\x12%\n\x0e\x65xample_string\x18\x01 \x01(\tR\rexampleString\x12\'\n\x0f\x65xample_integer\x18\x02 \x01(\x03R\x0e\x65xampleInteger2\xf2\x02\n\x04Test\x12V\n\x11\x45xampleUnaryUnary\x12\x1f.example_service.ExampleRequest\x1a .example_service.ExampleResponse\x12Y\n\x12\x45xampleUnaryStream\x12\x1f.example_service.ExampleRequest\x1a .example_service.ExampleResponse0\x01\x12Y\n\x12\x45xampleStreamUnary\x12\x1f.example_service.ExampleRequest\x1a .example_service.ExampleResponse(\x01\x12\\\n\x13\x45xampleStreamStream\x12\x1f.example_service.ExampleRequest\x1a .example_service.ExampleResponse(\x01\x30\x01\x42{\n\x13\x63om.example_serviceB\x0c\x45xampleProtoP\x01\xa2\x02\x03\x45XX\xaa\x02\x0e\x45xampleService\xca\x02\x0e\x45xampleService\xe2\x02\x1a\x45xampleService\\GPBMetadata\xea\x02\x0e\x45xampleServiceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'example_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023com.example_serviceB\014ExampleProtoP\001\242\002\003EXX\252\002\016ExampleService\312\002\016ExampleService\342\002\032ExampleService\\GPBMetadata\352\002\016ExampleService'
  _globals['_EXAMPLEREQUEST']._serialized_start=34
  _globals['_EXAMPLEREQUEST']._serialized_end=130
  _globals['_EXAMPLERESPONSE']._serialized_start=132
  _globals['_EXAMPLERESPONSE']._serialized_end=229
  _globals['_TEST']._serialized_start=232
  _globals['_TEST']._serialized_end=602
# @@protoc_insertion_point(module_scope)
